'use strict';

var LocalModulesHelpers = require('webpack/lib/dependencies/LocalModulesHelpers');
var LocalModuleDependency = require('webpack/lib/dependencies/LocalModuleDependency');
var OpenUI5DefineDependency = require('./OpenUI5DefineDependency');
var OpenUI5RequireItemDependency = require('./OpenUI5RequireItemDependency');

class OpenUI5DefineDependencyParserPlugin {
  constructor(options) {
    this.options = options;
  }

  newDefineDependency(range, arrayRange, functionRange, namedModule) {
    return new OpenUI5DefineDependency(range, arrayRange, functionRange, namedModule);
  }

  apply(parser) {
    var _this = this;

    parser.plugin('call sap.ui.define', function (expr) {
      var array = void 0;
      var fn = void 0;
      var namedModule = void 0;

      switch (expr.arguments.length) {
        case 1:
          fn = expr.arguments[0];
          break;
        case 2:
          if (expr.arguments[0].type === 'Literal') {
            namedModule = expr.arguments[0].value;
            fn = expr.arguments[1];
          } else if (expr.arguments[0].type === 'ArrayExpression') {
            array = expr.arguments[0];
            fn = expr.arguments[1];
          } else {
            fn = expr.arguments[0];
          }
          break;
        case 3:
          if (expr.arguments[0].type === 'Literal') {
            namedModule = expr.arguments[0].value;
            array = expr.arguments[1];
            fn = expr.arguments[2];
          } else {
            array = expr.arguments[0];
            fn = expr.arguments[1];
          }
          break;
        case 4:
          namedModule = expr.arguments[0].value;
          array = expr.arguments[1];
          fn = expr.arguments[2];
        // no default
      }

      if (array) {
        var param = parser.evaluateExpression(array);
        var result = parser.applyPluginsBailResult('call define:openui5:array', expr, param, namedModule);
        if (!result) return false;
      }

      if (fn && fn.type === 'FunctionExpression') {
        if (fn.body.type === 'BlockStatement') {
          parser.walkStatement(fn.body);
        } else {
          parser.walkExpression(fn.body);
        }
      }

      var dep = _this.newDefineDependency(expr.range, array ? array.range : null, fn ? fn.range : null, namedModule || null);
      dep.loc = expr.loc;
      if (namedModule) {
        dep.localModule = LocalModulesHelpers.addLocalModule(parser.state, namedModule);
      }
      parser.state.current.addDependency(dep);
      return true;
    });

    parser.plugin('call define:openui5:array', function (expr, param, namedModule) {
      param.items.forEach(function (param) {
        parser.applyPluginsBailResult('call define:openui5:item', expr, param, namedModule);
      });
      return true;
    });

    parser.plugin('call define:openui5:item', function (expr, param, namedModule) {
      var dep = void 0;
      var localModule = LocalModulesHelpers.getLocalModule(parser.state, param.string, namedModule);
      if (localModule) {
        dep = new LocalModuleDependency(localModule, param.range);
      } else {
        dep = new OpenUI5RequireItemDependency(param.string, param.range);
      }
      dep.loc = expr.loc;
      parser.state.current.addDependency(dep);
      return true;
    });
  }
}
module.exports = OpenUI5DefineDependencyParserPlugin;